﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_arms.lua
--
--    Copyright (C) 2014 jlongwell
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      jlongwell (Health@Blackrock)
------------------------------------------------------------------------------
-- Description:
--   Spec ability implementation for clcInfo_ArmsWarrior. 
------------------------------------------------------------------------------

if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Warrior then return end -- need clcInfo_Warrior
local _, englishClass = UnitClass("player") -- need to be a warrior
local arms = "Arms"
local _,specname = GetSpecializationInfo(GetActiveSpecGroup())

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Warrior = clcInfo_Warrior
local spells = clcInfo_Warrior.spells
local GetTimeLeft = clcInfo_Warrior.GetTimeLeft
local GetCooldown = clcInfo_Warrior.GetCooldown
local IconNext = clcInfo_Warrior.IconNext
local IconCurrent = clcInfo_Warrior.IconCurrent
local CalculateTargetHealthPct = clcInfo_Warrior.CalculateTargetHealthPct
local PickShout = clcInfo_Warrior.PickShout
local UnitPower = UnitPower
local UnitAura = UnitAura
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
local DELAY = clcInfo_Warrior.DELAY
local DISCOUNT = clcInfo_Warrior.DISCOUNT

-- Assign module variables
local specname = "Arms"
local prio_arg = "armsprio"
local sample_prio = "rend ravager level60 exe ms cs ww"
local priority = {"rend","ravager","level60","exe","ms","cs","ww"}
local class_module
local class_module_name = "Arms"

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Warrior.VERSION,
    priority = priority,
}
local tdb

-- actions ---------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action
-- functions up
local MortalStrikeId = spells["Mortal Strike"].id
local ColossusSmashId = spells["Colossus Smash"].id
local ExecuteId = spells["Execute"].id
local WhirlwindId = spells["Whirlwind"].id
local RendId = spells["Rend"].id
local DragonRoarId = spells["Dragon Roar"].id
local StormBoltId = spells["Storm Bolt"].id
local RavagerId = spells["Ravager"].id
local Enrage = spells["Enrage"].name
local MortalStrike =  spells["Mortal Strike"].name
local BattleShout = spells["Battle Shout"].name
local ColossusSmash = spells["Colossus Smash"].name
local CommandingShout = spells["Commanding Shout"].name
local Execute = spells["Execute"].name
local Rend = spells["Rend"].name
local SuddenDeathName = spells["Sudden Death"].id




local actions = {
    ms = {
        id = function()
            return MortalStrikeId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= MortalStrikeId and (rage > 20 or GetCSTime() > 0.1) and CalculateTargetHealthPct() > 20 then
                local _, _, _, cost = GetSpellInfo(MortalStrikeId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(MortalStrikeId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Mortal Strike"].name,
    },
	ravager = {
        id = function()
            return RavagerId
        end,
        GetCD = function(cur_id, rage)
			local _,talent_id = GetTalentRowSelectionInfo(7)
            if cur_id ~= RavagerId and (GetCSTime() < 0.1 and talent_id == 21205 and GetCooldown(RavagerId) < 5) then
                local _, _, _, cost = GetSpellInfo(RavagerId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(RavagerId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Ravager"].name,
    },
	level60 = {
        id = function()
            return GetTalent()
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= GetTalent() and GetCooldown(MortalStrikeId) > 0 and GetCooldown(GetTalent()) < 5 then
                local _, _, _, cost = GetSpellInfo(GetTalent())
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(GetTalent()) + add_time
            end
            return DISCOUNT
        end,
        info = "Level 60 Talent",
    },
    rend = {
        id = function()
            return RendId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= RendId and (GetRendTime() < 5 and GetCSTime() < 0.1) then
                local _, _, _, cost = GetSpellInfo(RendId)
                local add_time = ((cost > rage) and DELAY) or 0
				--print(GetRendTime())
                return GetCooldown(RendId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Rend"].name,
    },
    cs = {
        id = function()
            return ColossusSmashId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= ColossusSmashId 
			   and GetCooldown(ColossusSmashId) < 4 then
                local _, _, _, cost = GetSpellInfo(ColossusSmashId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(ColossusSmashId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Colossus Smash"].name,
    },
    exe = {
        id = function()
            return ExecuteId
        end,
        GetCD = function(_, rage)
            if (CalculateTargetHealthPct() <= 20 and rage >= 40)
			or (SuddenDeath() == true and GetCooldown(ColossusSmashId) > 0.5) then
                local _, _, _, cost = GetSpellInfo(ExecuteId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(ExecuteId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Execute"].name,
    },
	ww = {
        id = function()
            return WhirlwindId
        end,
        GetCD = function(_, rage)
            if (rage >= 40 and GetCooldown(ColossusSmashId) > 5) 
			or (GetCooldown(MortalStrikeId) > 1 and GetCSTime() > 0.1)
			or rage > 60
			then
                local _, _, _, cost = GetSpellInfo(WhirlwindId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(WhirlwindId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Whirlwind"].name,
    },
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB
local GetPriority
local SetPriority

function GetTalent()
	local _, talent_id = GetTalentRowSelectionInfo(4)
	if talent_id == 16037 then
		return 118000
	elseif talent_id == 15759 then
		return 107570
	else
		return 46968
	end
end

function SuddenDeath()
	local expire = GetTimeLeft("player",SuddenDeathName,"HELPFUL|PLAYER")
	if expire > 0 then
		return true
	else
		return false
	end
end

function GetCSTime()
	local _,_,_,_,_,_,expire = UnitAura("target",ColossusSmash, nil, "PLAYER|HARMFUL")
	if expire ~= nil then
		return (expire - GetTime())
	else
		return 0
	end
end

function GetRendTime()
	local _,_,_,_,_,_,expire = UnitAura("target",Rend, nil, "PLAYER|HARMFUL")
	if expire ~= nil then
		return (expire - GetTime())
	else
		return 0
	end
end
--initialize module.
class_module, class_module_name = clcInfo_Warrior:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
    tdb, GetTDB, SetTDB, GetPriority, SetPriority
        = clcInfo_Warrior:OnTemplatesUpdate(class_module_name, actions,
                                          tdb_default, specname)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db,GetDB,SetDB = clcInfo_Warrior:OnInitialize(class_module_name, db_default)
end

-- called when module options are opened
local function LoadArmsModule()
    local talent_tree_name = "Arms"
    clcInfo_Warrior.LoadModule(class_module_name, talent_tree_name, sample_prio,
                               actions, GetDB, SetDB, GetTDB, SetTDB,
                               GetPriority, SetPriority)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadArmsModule

-- giving queue as command line
local function CmdArms(args)
    clcInfo_Warrior:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdArms

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconArmsCurrent(...)
    return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconArmsNext(...)
    return IconNext()
end
