﻿------------------------------------------------------------------------------
-- $Id$
------------------------------------------------------------------------------
--                            module_fury.lua
--
--    Copyright (C) 2014 jlongwell
--
--  This program is free software: you can redistribute it and/or modify
--  it under the terms of the GNU Lesser General Public License as published
--  by the Free Software Foundation, either version 3 of the License, or
--  (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
--  You should have received a copy of the GNU Lesser General Public License
--  along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
--  This file is for use with clcInfo by abija.
--
--  abija has distributed clcInfo All Rights Reserved.  For more information
--  go to: http://wow.curseforge.com/addons/clctracker/
--
------------------------------------------------------------------------------
-- Author:
--      jlongwell (Health@Blackrock)
------------------------------------------------------------------------------
-- Description:
--   Spec ability implementation for clcInfo_Warrior. 
------------------------------------------------------------------------------

if not clcInfo then return end       -- need clcInfo to run
if not clcInfo_Warrior then return end -- need clcInfo_Warrior
local _, englishClass = UnitClass("player") -- need to be a warrior
local _, englishClass = UnitClass("player") -- need to be a warrior
local fury = "Fury"
local _,specname = GetSpecializationInfo(GetActiveSpecGroup())

-- Get local copies of functions/tables.  This will speed up execution.
local clcInfo_Warrior = clcInfo_Warrior
local spells = clcInfo_Warrior.spells
local GetTimeLeft = clcInfo_Warrior.GetTimeLeft
local GetCooldown = clcInfo_Warrior.GetCooldown
local IconNext = clcInfo_Warrior.IconNext
local IconCurrent = clcInfo_Warrior.IconCurrent
local CalculateTargetHealthPct = clcInfo_Warrior.CalculateTargetHealthPct
local PickShout = clcInfo_Warrior.PickShout
local UnitPower = UnitPower
local UnitAura = UnitAura
local GetNumPartyMembers = GetNumPartyMembers
local GetNumRaidMembers = GetNumRaidMembers
local DELAY = clcInfo_Warrior.DELAY
local DISCOUNT = clcInfo_Warrior.DISCOUNT

-- Assign module variables
local specname = "Fury"
local prio_arg = "furyprio"
local sample_prio = "br exe ws bt level60 ravager rb"
local priority = {"br","exe","ws","bt","level60","ravager","rb"}
local class_module
local class_module_name = "Fury"

local db_default = {
    char_var = false,
}
local db

local tdb_default = {
    version = clcInfo_Warrior.VERSION,
    priority = priority,
}
local tdb

-- actions ---------------------------------------------------------------------
-- local copies of spell names and ids to ugly up code but speed action
-- functions up
local BloodthirstId = spells["Bloodthirst"].id
local ExecuteId = spells["Execute"].id
local WhirlwindId = spells["Whirlwind"].id
local RavagerId = spells["Ravager"].id
local EnrageId = spells["Enrage"].id
local WildStrikeId = spells["Wild Strike"].id
local RagingBlowId = spells["Raging Blow"].id
local BerserkerRageId = spells["Berserker Rage"].id
local SuddenDeathName = spells["Sudden Death"].name
local Enrage = spells["Enrage"].name
local RagingBlow = spells["Raging Blow"].name
local Bloodsurge = spells["Bloodsurge"].name
local RB = spells["Raging Blow!"].name


local actions = {
    bt = {
        id = function()
            return BloodthirstId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= BloodthirstId 
			and not (Enraged() == true and GetRBStacks() > 1 )
			then
                local _, _, _, cost = GetSpellInfo(BloodthirstId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(BloodthirstId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Bloodthirst"].name,
    },
	rb = {
        id = function()
            return RagingBlowId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= RagingBlowId
			   and (CalculateTargetHealthPct() >= 20 and (GetRBStacks() > 1 or (GetCooldown(BloodthirstId) > 0 and GetCooldown(GetTalent()) > 0 and GetRBStacks() > 0)))
			   or (CalculateTargetHealthPct() <= 20 and (GetRBStacks() > 1 or (GetCooldown(BloodthirstId) > 0 and GetCooldown(GetTalent()) > 0 and GetRBStacks() > 0 and rage < 30))) then
                local _, _, _, cost = GetSpellInfo(RagingBlowId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(RagingBlowId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Raging Blow"].name,
    },
	ws = {
        id = function()
            return WildStrikeId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= WildStrikeId
               and (CalculateTargetHealthPct() >= 20 and ((GetTimeLeft("player","Bloodsurge","HELPFUL") >= .1) or (GetRBStacks() < 2 and rage >90)))
			   or (CalculateTargetHealthPct() <= 20 and (GetTimeLeft("player","Bloodsurge","HELPFUL") >= .1))then
                local _, _, _, cost = GetSpellInfo(WildStrikeId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(WildStrikeId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Wild Strike"].name,
    },
    exe = {
        id = function()
            return ExecuteId
        end,
        GetCD = function(_, rage)
            if SuddenDeath() == true
			or (CalculateTargetHealthPct() <= 20 and Enraged() == true and rage > 30)
			or (CalculateTargetHealthPct() <= 20 and rage > 40)
			then
                local _, _, _, cost = GetSpellInfo(ExecuteId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(ExecuteId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Execute"].name,
    },
	level60 = {
        id = function()
            return GetTalent()
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= GetTalent() 
			and GetCooldown(BloodthirstId) > 0 
			and GetCooldown(GetTalent()) < 3 
			and GetRBStacks() < 2 then
                local _, _, _, cost = GetSpellInfo(GetTalent())
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(GetTalent()) + add_time
            end
            return DISCOUNT
        end,
        info = "Level 60 Talent",
    },
	br = {
        id = function()
            return BerserkerRageId
        end,
        GetCD = function(cur_id, rage)
            if cur_id ~= BerserkerRageId  
			and (Enraged() == false and GetCooldown(BloodthirstId) > 0)then
                local _, _, _, cost = GetSpellInfo(BerserkerRageId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(BerserkerRageId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Berserker Rage"].name,
    },
	ravager = {
        id = function()
            return RavagerId
        end,
        GetCD = function(cur_id, rage)
			local _,talent_id = GetTalentRowSelectionInfo(7)
            if cur_id ~= RavagerId 
			and (GetCooldown(BloodthirstId) > 0 and rage < 90 and GetRBStacks() < 2)
			and talent_id == 21205 then
                local _, _, _, cost = GetSpellInfo(RavagerId)
                local add_time = ((cost > rage) and DELAY) or 0
                return GetCooldown(RavagerId) + add_time
            end
            return DISCOUNT
        end,
        info = spells["Ravager"].name,
    },
}

----FUNCTIONS-------------------------------------------------------------------
-- The following are called once or infrequently, so speed isn't an issue.

-- called whenever module options are loaded
local GetDB
local SetDB
local GetTDB
local SetTDB
local GetPriority
local SetPriority

function GetTalent()
	local _, talent_id = GetTalentRowSelectionInfo(4)
	if talent_id == 16037 then
		return 118000
	elseif talent_id == 15759 then
		return 107570
	else
		return 46968
	end
end

function SuddenDeath()
	local expire = GetTimeLeft("player",SuddenDeathName,"HELPFUL|PLAYER")
	if expire > 0 then
		return true
	else
		return false
	end
end

function Enraged()
	local expire = GetTimeLeft("player",Enrage,"HELPFUL|PLAYER")
	if expire > 0 then
		return true
	else
		return false
	end
end

function GetRBStacks()
	local _,_,_,rbstacks = UnitAura("player", RB)
	if rbstacks ~= nil then
		return rbstacks
	else
		return 0
	end
end

--initialize module.
class_module, class_module_name = clcInfo_Warrior:init_module(specname)

--called when template updates.
function class_module:OnTemplatesUpdate()
    tdb, GetTDB, SetTDB, GetPriority, SetPriority
        = clcInfo_Warrior:OnTemplatesUpdate(class_module_name, actions,
                                          tdb_default, specname)
end

-- called each time clcInfo initializes
function class_module.OnInitialize()
    db,GetDB,SetDB = clcInfo_Warrior:OnInitialize(class_module_name, db_default)
end

-- called when module options are opened
local function LoadFuryModule()
    local talent_tree_name = "Fury"
    clcInfo_Warrior.LoadModule(class_module_name, talent_tree_name, sample_prio,
                               actions, GetDB, SetDB, GetTDB, SetTDB,
                               GetPriority, SetPriority)
end
clcInfo.optionsCMLoaders[#(clcInfo.optionsCMLoaders) + 1] = LoadFuryModule

-- giving queue as command line
local function CmdArms(args)
    clcInfo_Warrior:CmdLine(tdb, args, actions, specname)
end

clcInfo.cmdList[prio_arg] = CmdArms

----ABILITY ICONS--------------------------------------------------------------
----Add these to the clcInfo.env (so they live with IconAura, etc.
local emod = clcInfo.env
function emod.IconFuryCurrent(...)
    return IconCurrent(actions, specname, tdb.rangePerSkill)
end
function emod.IconFuryNext(...)
    return IconNext()
end
